@@ -26,20 +26,20 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 
 import os
+import sys
 import xml
 import urllib
 import urlparse
 import threading
 import zipfile
-import gnomekeyring as keyring
 
 import rb
 from gi.repository import RB
-from gi.repository import GObject, Gtk, Gio
-# XXX use GnomeKeyring when introspection is available
+from gi.repository import GObject, Gtk, Gdk, Gio
 
 from TrackListHandler import TrackListHandler
-from BuyAlbumHandler import BuyAlbumHandler, MagnatunePurchaseError
+from DownloadAlbumHandler import DownloadAlbumHandler, MagnatuneDownloadError
+import MagnatuneAccount
 
 import gettext
 gettext.install('rhythmbox', RB.locale_dir())
@@ -47,7 +47,8 @@ gettext.install('rhythmbox', RB.locale_dir())
 magnatune_partner_id = "rhythmbox"
 
 # URIs
-magnatune_song_info_uri = Gio.file_new_for_uri("http://magnatune.com/info/song_info_xml.zip")
+magnatune_song_info_uri = "http://magnatune.com/info/song_info_xml.zip"
+magnatune_changed_uri = "http://magnatune.com/info/changed.txt"
 magnatune_buy_album_uri = "https://magnatune.com/buy/choose?"
 magnatune_api_download_uri = "http://%s:%s@download.magnatune.com/buy/membership_free_dl_xml?"
 
@@ -56,6 +57,7 @@ magnatune_cache_dir = Gio.file_new_for_path(RB.user_cache_dir()).resolve_relativ
 
 magnatune_song_info = os.path.join(magnatune_cache_dir.get_path(), 'song_info.xml')
 magnatune_song_info_temp = os.path.join(magnatune_cache_dir.get_path(), 'song_info.zip.tmp')
+magnatune_changes = os.path.join(magnatune_cache_dir.get_path(), 'changed.txt')
 
 
 class MagnatuneSource(RB.BrowserSource):
@@ -85,9 +87,9 @@ class MagnatuneSource(RB.BrowserSource):
 
 		# album download stuff
 		self.__downloads = {} # keeps track of download progress for each file
-		self.__cancellables = {} # keeps track of Gio.Cancellable objects so we can abort album downloads
+		self.__copies = {} # keeps copy objects for each file
 
-		self.__art_store = RB.ExtDB("album-art")
+		self.__art_store = RB.ExtDB(name="album-art")
 
 	#
 	# RBSource methods
@@ -115,11 +117,6 @@ class MagnatuneSource(RB.BrowserSource):
 			qm = self.props.query_model
 			return (qm.compute_status_normal("%d song", "%d songs"), None, 2.0)
 
-	def do_get_ui_actions(self):
-		return ["MagnatuneDownloadAlbum",
-			"MagnatuneArtistInfo",
-			"MagnatuneCancelDownload"]
-
 	def do_selected(self):
 		if not self.__activated:
 			shell = self.props.shell
@@ -144,10 +141,10 @@ class MagnatuneSource(RB.BrowserSource):
 	def do_impl_can_delete(self):
 		return False
 
-	def do_impl_pack_paned(self, paned):
+	def do_pack_content(self, content):
 		self.__paned_box = Gtk.VBox(homogeneous=False, spacing=5)
 		self.pack_start(self.__paned_box, True, True, 0)
-		self.__paned_box.pack_start(paned, True, True, 0)
+		self.__paned_box.pack_start(content, True, True, 0)
 
 
 	def do_delete_thyself(self):
@@ -179,28 +176,28 @@ class MagnatuneSource(RB.BrowserSource):
 		urls = set([])
 
 		for tr in tracks:
-			sku = self.__sku_dict[self.__db.entry_get_string(tr, RB.RhythmDBPropType.LOCATION)]
+			sku = self.__sku_dict[tr.get_string(RB.RhythmDBPropType.LOCATION)]
 			url = self.__home_dict[sku]
 			if url not in urls:
 				Gtk.show_uri(screen, url, Gdk.CURRENT_TIME)
 				urls.add(url)
 
-	def purchase_redirect(self):
+	def download_redirect(self):
 		screen = self.props.shell.props.window.get_screen()
 		tracks = self.get_entry_view().get_selected_entries()
 		urls = set([])
 
 		for tr in tracks:
-			sku = self.__sku_dict[self.__db.entry_get_string(tr, RB.RhythmDBPropType.LOCATION)]
+			sku = self.__sku_dict[tr.get_string(RB.RhythmDBPropType.LOCATION)]
 			url = magnatune_buy_album_uri + urllib.urlencode({ 'sku': sku, 'ref': magnatune_partner_id })
 			if url not in urls:
 				Gtk.show_uri(screen, url, Gdk.CURRENT_TIME)
 				urls.add(url)
 
 	def download_album(self):
-		if selt.__settings['account_type'] != 'download':
-			# The user doesn't have a download account, so redirect them to the purchase page.
-			self.purchase_redirect()
+		if self.__settings['account-type'] != 'download':
+			# The user doesn't have a download account, so redirect them to the download signup page
+			self.download_redirect()
 			return
 
 		try:
@@ -208,15 +205,15 @@ class MagnatuneSource(RB.BrowserSource):
 			library = Gio.Settings("org.gnome.rhythmbox.rhythmdb")
 			library_location = library['locations'][0]
 		except IndexError, e:
-			RB.error_dialog(title = _("Couldn't purchase album"),
-				        message = _("You must have a library location set to purchase an album."))
+			RB.error_dialog(title = _("Couldn't download album"),
+				        message = _("You must have a library location set to download an album."))
 			return
 
 		tracks = self.get_entry_view().get_selected_entries()
 		skus = []
 
 		for track in tracks:
-			sku = self.__sku_dict[self.__db.entry_get_string(track, RB.RhythmDBPropType.LOCATION)]
+			sku = self.__sku_dict[track.get_string(RB.RhythmDBPropType.LOCATION)]
 			if sku in skus:
 				continue
 			skus.append(sku)
@@ -227,9 +224,24 @@ class MagnatuneSource(RB.BrowserSource):
 	#
 
 	def __update_catalogue(self):
-		def update_cb(result):
+		def update_cb(remote_changes):
 			self.__catalogue_check = None
-			if result is True:
+			try:
+				f = open(magnatune_changes, 'r')
+				local_changes = f.read().strip()
+			except:
+				local_changes = ""
+
+			remote_changes = remote_changes.strip()
+			print "local checksum %s, remote checksum %s" % (local_changes, remote_changes)
+			if local_changes != remote_changes:
+				try:
+					f = open(magnatune_changes, 'w')
+					f.write(remote_changes + "\n")
+					f.close()
+				except Exception, e:
+					print "unable to write local change id: %s" % str(e)
+
 				download_catalogue()
 			elif self.__has_loaded is False:
 				load_catalogue()
@@ -241,19 +253,17 @@ class MagnatuneSource(RB.BrowserSource):
 						return info.filename;
 				return None
 
-			def download_progress(complete, total):
+			def download_progress(copy, complete, total, self):
 				self.__load_progress = (complete, total)
 				self.__notify_status_changed()
 
-			def download_finished(uri, result):
-				try:
-					success = uri.copy_finish(result)
-				except:
-					success = False
-
+			def download_finished(copy, success, self):
 				if not success:
+					print "catalog download failed"
+					print copy.get_error()
 					return
 
+				print "catalog download successful"
 				# done downloading, unzip to real location
 				catalog_zip = zipfile.ZipFile(magnatune_song_info_temp)
 				catalog = open(magnatune_song_info, 'w')
@@ -266,7 +276,8 @@ class MagnatuneSource(RB.BrowserSource):
 				catalog.close()
 				catalog_zip.close()
 
-				dest.delete()
+				df = Gio.file_new_for_path(magnatune_song_info_temp)
+				df.delete(None)
 				self.__updating = False
 				self.__catalogue_loader = None
 				self.__notify_status_changed()
@@ -276,46 +287,23 @@ class MagnatuneSource(RB.BrowserSource):
 
 			self.__updating = True
 
-			dest = Gio.file_new_for_path(magnatune_song_info_temp)
-			self.__catalogue_loader = Gio.Cancellable()
 			try:
-				# For some reason, Gio.FileCopyFlags.OVERWRITE doesn't work for copy_async
-				dest.delete()
+				df = Gio.file_new_for_path(magnatune_song_info_temp)
+				df.delete(None)
 			except:
 				pass
-			magnatune_song_info_uri.copy_async(dest,
-			                                   download_finished,
-							   progress_callback=download_progress,
-							   flags=Gio.FileCopyFlags.OVERWRITE,
-							   cancellable=self.__catalogue_loader)
+			self.__catalog_loader = RB.AsyncCopy()
+			self.__catalog_loader.set_progress(download_progress, self)
+			self.__catalog_loader.start(magnatune_song_info_uri, magnatune_song_info_temp, download_finished, self)
 
 		def load_catalogue():
-			def got_items(result, items):
-				account_type = self.__settings['account_type']
-				username = ""
-				password = ""
-				if account_type == 'none':
-					pass
-				elif result is not None or len(items) == 0:
-					RB.error_dialog(title = _("Couldn't get account details"),
-							message = str(result))
-					return
-				else:
-					try:
-						username, password = items[0].secret.split('\n')
-					except ValueError: # Couldn't parse secret, possibly because it's empty
-						pass
-				parser = xml.sax.make_parser()
-				parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__art_dict, account_type, username, password))
 
-				self.__catalogue_loader = rb.ChunkLoader()
-				self.__catalogue_loader.get_url_chunks(magnatune_song_info, 64*1024, True, catalogue_chunk_cb, parser)
-
-			def catalogue_chunk_cb(result, total, parser):
-				if not result or isinstance(result, Exception):
-					if result:
+			def catalogue_chunk_cb(loader, data, total, parser):
+				if data is None:
+					error = loader.get_error()
+					if error:
 						# report error somehow?
-						print "error loading catalogue: %s" % result
+						print "error loading catalogue: %s" % error
 
 					try:
 						parser.close()
@@ -329,27 +317,28 @@ class MagnatuneSource(RB.BrowserSource):
 
 					# restart in-progress downloads
 					# (doesn't really belong here)
-					for f in magnatune_in_progress_dir.enumerate_children('standard::name'):
+					for f in magnatune_in_progress_dir.enumerate_children('standard::name', Gio.FileQueryInfoFlags.NONE, None):
 						name = f.get_name()
 						if not name.startswith("in_progress_"):
 							continue
-						uri = magnatune_in_progress_dir.resolve_relative_path(name).load_contents()[0]
+						(result, uri, etag) = magnatune_in_progress_dir.resolve_relative_path(name).load_contents(None)
 						print "restarting download from %s" % uri
-						self.__download_album(Gio.file_new_for_uri(uri), name[12:])
+						self.__download_album(uri, name[12:])
 				else:
 					# hack around some weird chars that show up in the catalogue for some reason
-					result = result.replace("\x19", "'")
-					result = result.replace("\x13", "-")
+					data = str(data.str)
+					data = data.replace("\x19", "'")
+					data = data.replace("\x13", "-")
 
 					# argh.
-					result = result.replace("Rock & Roll", "Rock &amp; Roll")
+					data = data.replace("Rock & Roll", "Rock &amp; Roll")
 
 					try:
-						parser.feed(result)
+						parser.feed(data)
 					except xml.sax.SAXParseException, e:
 						print "error parsing catalogue: %s" % e
 
-					load_size['size'] += len(result)
+					load_size['size'] += len(data)
 					self.__load_progress = (load_size['size'], total)
 
 				self.__notify_status_changed()
@@ -361,11 +350,17 @@ class MagnatuneSource(RB.BrowserSource):
 			self.__notify_status_changed()
 
 			load_size = {'size': 0}
-			keyring.find_items(keyring.ITEM_GENERIC_SECRET, {'rhythmbox-plugin': 'magnatune'}, got_items)
+
+			parser = xml.sax.make_parser()
+			parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__art_dict))
+
+			self.__catalogue_loader = RB.ChunkLoader()
+			self.__catalogue_loader.set_callback(catalogue_chunk_cb, parser)
+			self.__catalogue_loader.start(magnatune_song_info, 64*1024)
 
 
-		self.__catalogue_check = rb.UpdateCheck()
-		self.__catalogue_check.check_for_update(magnatune_song_info, magnatune_song_info_uri.get_uri(), update_cb)
+		self.__catalogue_check = rb.Loader()
+		self.__catalogue_check.get_url(magnatune_changed_uri, update_cb)
 
 
 	def __show_loading_screen(self, show):
@@ -395,32 +390,11 @@ class MagnatuneSource(RB.BrowserSource):
 	#
 
 	def __auth_download(self, sku): # http://magnatune.com/info/api
-		def got_items(result, items):
-			if result is not None or len(items) == 0:
-				RB.error_dialog(title = _("Couldn't get account details"),
-				                message = str(result))
-				return
-
-			try:
-				username, password = items[0].secret.split('\n')
-			except ValueError: # Couldn't parse secret, possibly because it's empty
-				username = ""
-				password = ""
-			print "downloading album: " + sku
-			url_dict = {
-				'id':	magnatune_partner_id,
-				'sku':	sku
-			}
-			url = magnatune_api_download_uri % (username, password)
-			url = url + urllib.urlencode(url_dict)
-
-			l = rb.Loader()
-			l.get_url(url, auth_data_cb, (username, password))
 
 		def auth_data_cb(data, (username, password)):
-			buy_album_handler = BuyAlbumHandler(self.__settings['format'])
+			dl_album_handler = DownloadAlbumHandler(self.__settings['format'])
 			auth_parser = xml.sax.make_parser()
-			auth_parser.setContentHandler(buy_album_handler)
+			auth_parser.setContentHandler(dl_album_handler)
 
 			if data is None:
 				# hmm.
@@ -428,12 +402,13 @@ class MagnatuneSource(RB.BrowserSource):
 
 			try:
 				data = data.replace("<br>", "") # get rid of any stray <br> tags that will mess up the parser
+				data = data.replace(" & ", " &amp; ") # clean up some missing escaping
 				# print data
 				auth_parser.feed(data)
 				auth_parser.close()
 
 				# process the URI: add authentication info, quote the filename component for some reason
-				parsed = urlparse.urlparse(buy_album_handler.url)
+				parsed = urlparse.urlparse(dl_album_handler.url)
 				netloc = "%s:%s@%s" % (username, password, parsed.hostname)
 
 				spath = os.path.split(urllib.url2pathname(parsed.path))
@@ -443,33 +418,41 @@ class MagnatuneSource(RB.BrowserSource):
 				authed = (parsed[0], netloc, path) + parsed[3:]
 				audio_dl_uri = urlparse.urlunparse(authed)
 
-				self.__download_album(Gio.file_new_for_uri(audio_dl_uri), sku)
+				print "download uri for %s is %s" % (sku, audio_dl_uri)
+				self.__download_album(audio_dl_uri, sku)
 
-			except MagnatunePurchaseError, e:
+			except MagnatuneDownloadError, e:
 				RB.error_dialog(title = _("Download Error"),
 						message = _("An error occurred while trying to authorize the download.\nThe Magnatune server returned:\n%s") % str(e))
 			except Exception, e:
+				sys.excepthook(*sys.exc_info())
 				RB.error_dialog(title = _("Error"),
 						message = _("An error occurred while trying to download the album.\nThe error text is:\n%s") % str(e))
 
+		print "downloading album: " + sku
+		account = MagnatuneAccount.instance()
+		(account_type, username, password) = account.get()
+		url_dict = {
+			'id':	magnatune_partner_id,
+			'sku':	sku
+		}
+		url = magnatune_api_download_uri % (username, password)
+		url = url + urllib.urlencode(url_dict)
+
+		l = rb.Loader()
+		l.get_url(url, auth_data_cb, (username, password))
 
-		keyring.find_items(keyring.ITEM_GENERIC_SECRET, {'rhythmbox-plugin': 'magnatune'}, got_items)
 
 	def __download_album(self, audio_dl_uri, sku):
-		def download_progress(current, total):
-			self.__downloads[str_uri] = (current, total)
+		def download_progress(copy, complete, total, self):
+			self.__downloads[audio_dl_uri] = (complete, total)
 			self.__notify_status_changed()
 
-		def download_finished(uri, result):
-			del self.__cancellables[str_uri]
-			del self.__downloads[str_uri]
-
-			try:
-				success = uri.copy_finish(result)
-			except Exception, e:
-				success = False
-				print "Download not completed: " + str(e)
+		def download_finished(copy, success, self):
+			del self.__downloads[audio_dl_uri]
+			del self.__copies[audio_dl_uri]
 
+			print "download of %s finished: %s" % (audio_dl_uri, success)
 			if success:
 				threading.Thread(target=unzip_album).start()
 			else:
@@ -480,7 +463,7 @@ class MagnatuneSource(RB.BrowserSource):
 				manager = shell.props.ui_manager
 				manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(False)
 				if success:
-					shell.notify_custom(4000, _("Finished Downloading"), _("All Magnatune downloads have been completed."))
+					shell.notify_custom(4000, _("Finished Downloading"), _("All Magnatune downloads have been completed."), None, False)
 
 			self.__notify_status_changed()
 
@@ -489,58 +472,59 @@ class MagnatuneSource(RB.BrowserSource):
 			library = Gio.Settings("org.gnome.rhythmbox.rhythmdb")
 			library_location = Gio.file_new_for_uri(library['locations'][0])
 
+			print "unzipping %s" % dest.get_path()
 			album = zipfile.ZipFile(dest.get_path())
 			for track in album.namelist():
 				track_uri = library_location.resolve_relative_path(track).get_uri()
+				print "zip file entry: %s => %s" % (track, track_uri)
 
 				track_uri = RB.sanitize_uri_for_filesystem(track_uri)
 				RB.uri_create_parent_dirs(track_uri)
 
-				track_out = Gio.file_new_for_uri(track_uri).create()
+				track_out = Gio.file_new_for_uri(track_uri).create(Gio.FileCreateFlags.NONE, None)
 				if track_out is not None:
-					track_out.write(album.read(track))
-					track_out.close()
+					track_out.write(album.read(track), None)
+					track_out.close(None)
+					print "adding %s to library" % track_uri
 					self.__db.add_uri(track_uri)
 
 			album.close()
 			remove_download_files()
 
 		def remove_download_files():
-			in_progress.delete()
-			dest.delete()
-
+			print "removing download files"
+			in_progress.delete(None)
+			dest.delete(None)
 
 		in_progress = magnatune_in_progress_dir.resolve_relative_path("in_progress_" + sku)
 		dest = magnatune_in_progress_dir.resolve_relative_path(sku)
 
-		str_uri = audio_dl_uri.get_uri()
-		in_progress.replace_contents(str_uri, None, False, flags=Gio.FileCreateFlags.PRIVATE|Gio.FileCreateFlags.REPLACE_DESTINATION)
+		in_progress.replace_contents(str(audio_dl_uri),
+					     None,
+					     False,
+					     Gio.FileCreateFlags.PRIVATE|Gio.FileCreateFlags.REPLACE_DESTINATION,
+					     None)
 
 		shell = self.props.shell
 		manager = shell.props.ui_manager
 		manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(True)
 
-		self.__downloads[str_uri] = (0, 0) # (current, total)
-
-		cancel = Gio.Cancellable()
-		self.__cancellables[str_uri] = cancel
 		try:
 			# For some reason, Gio.FileCopyFlags.OVERWRITE doesn't work for copy_async
-			dest.delete()
+			dest.delete(None)
 		except:
 			pass
 
-		# no way to resume downloads, sadly
-		audio_dl_uri.copy_async(dest,
-		                        download_finished,
-					progress_callback=download_progress,
-					flags=Gio.FileCopyFlags.OVERWRITE,
-					cancellable=cancel)
+		dl = RB.AsyncCopy()
+		dl.set_progress(download_progress, self)
+		dl.start(audio_dl_uri, dest.get_uri(), download_finished, self)
+		self.__downloads[audio_dl_uri] = (0, 0) # (current, total)
+		self.__copies[audio_dl_uri] = dl
 
 
 	def cancel_downloads(self):
-		for cancel in self.__cancellables.values():
-			cancel.cancel()
+		for download in self.__copies.values():
+			download.cancel()
 
 		shell = self.props.shell
 		manager = shell.props.ui_manager
